@@ -38,31 +38,6 @@ LOGGER = logging.getLogger(__name__)
 NOSQUARE = 64
 NOINDEX = -1
 
-WHITES = 1 << 6
-BLACKS = 1 << 7
-
-NOPIECE = 0
-PAWN = 1
-KNIGHT = 2
-BISHOP = 3
-ROOK = 4
-QUEEN = 5
-KING = 6
-
-wK = KING | WHITES
-wP = PAWN | WHITES
-wN = KNIGHT | WHITES
-wB = BISHOP | WHITES
-wR = ROOK | WHITES
-wQ = QUEEN | WHITES
-
-bK = KING | BLACKS
-bP = PAWN | BLACKS
-bN = KNIGHT | BLACKS
-bB = BISHOP | BLACKS
-bR = ROOK | BLACKS
-bQ = QUEEN | BLACKS
-
 MAX_KKINDEX = 462
 MAX_PPINDEX = 576
 MAX_PpINDEX = 24 * 48
@@ -128,6 +103,9 @@ def map24_b(s):
     s = s - 8
     return ((s & 3) + s) >> 1
 
+def map88(x):
+    return x + (x & 56)
+
 def in_queenside(x):
     return (x & (1 << 2)) == 0
 
@@ -144,35 +122,29 @@ def idx_is_empty(x):
     return x == -1
 
 
-def getcol(x):
-    return x & 7
-
-def getrow(x):
-    return x >> 3
-
 def flip_type(x, y):
     ret = 0
 
-    if getcol(x) > 3:
+    if chess.square_file(x) > 3:
         x = flip_we(x)
         y = flip_we(y)
         ret |= 1
 
-    if getrow(x) > 3:
+    if chess.square_rank(x) > 3:
         x = flip_ns(x)
         y = flip_ns(y)
         ret |= 2
 
-    rowx = getrow(x)
-    colx = getcol(x)
+    rowx = chess.square_rank(x)
+    colx = chess.square_file(x)
 
     if rowx > colx:
         x = flip_nw_se(x)
         y = flip_nw_se(y)
         ret |= 4
 
-    rowy = getrow(y)
-    coly = getcol(y)
+    rowy = chess.square_rank(y)
+    coly = chess.square_file(y)
     if rowx == colx and rowy > coly:
         x = flip_nw_se(x)
         y = flip_nw_se(y)
@@ -396,197 +368,24 @@ def init_ppidx():
 PPIDX, PP_HI24, PP_LO48 = init_ppidx()
 
 
-def bb_isbiton(bb, bit):
-        return 0 != (bb >> bit) & 1
-
-def map88(x):
-        return x + (x & 56)
-
-def unmap88(x):
-        return x + (x & 7) >> 1
-
-def mapx88(x):
-        return ((x & 56) << 1) | (x & 7)
-
-BSTEP = [17, 15, -15, -17, 0]
-RSTEP = [1, 16, -1, -16, 0]
-NSTEP = [18, 33, 31, 14, -18, -33, -31, -14, 0]
-KSTEP = [1, 17, 16, 15, -1, -17, -16, -15, 0]
-
-PSTEPARR = [
-    None,  # No piece.
-    None,  # Pawn.
-    NSTEP,
-    BSTEP,
-    RSTEP,
-    KSTEP,  # Queen.
-    KSTEP,  # King.
-]
-
-PSLIDER = [
-    False,  # No piece.
-    False,  # Pawn.
-    False,
-    True,
-    True,
-    True,
-    False,
-]
-
-def gen_rev(occ, input_piece, sq):
-    # Get list of reversible piece moves. Yields squares.
-    from_ = map88(sq)
-
-    pc = input_piece & (PAWN | KNIGHT | BISHOP | ROOK | QUEEN | KING)
-
-    steparr = PSTEPARR[pc]
-    slider = PSLIDER[pc]
-
-    if slider:
-        for step in steparr:
-            if step == 0:
-                break
-            s = from_ + step
-            while 0 == (s & 0x88):
-                us = unmap88(s)
-                if 0 != (0x1 & (occ >> us)):
-                    break
-                yield us
-                s += step
-
-    else:
-        for step in steparr:
-            if step == 0:
-                break
-            s = from_ + step
-            if 0 == (s & 0x88):
-                us = unmap88(s)
-                if 0 == (0x1 & (occ >> us)):
-                    yield us
-
-def reach_init():
-    stp_a = [15, -15]
-    stp_b = [17, -17]
-    reach = [[-1] * 64 for _ in range(7)]
-
-    for pc in range(KNIGHT, KING + 1):
-        for sq in range(64):
-            bb = 0
-            for li in gen_rev(0, pc, sq):
-                bb |= 1 << li
-                reach[pc][sq] = bb
-
-    for side in range(2):
-        index = 1 ^ side
-        step_a = stp_a[side]
-        step_b = stp_b[side]
-        for sq in range(64):
-            sq88 = map88(sq)
-            bb = 0
-
-            thelist = []
-
-            s = sq88 + step_a
-            if 0 == (s & 0x88):
-                us = unmap88(s)
-                thelist.append(us)
-
-            s = sq88 + step_b
-            if 0 == (s & 0x88):
-                us = unmap88(s)
-                thelist.append(us)
-
-            for li in thelist:
-                bb |= 1 << li
-
-            reach[index][sq] = bb
-
-    return reach
-
-REACH = reach_init()
-
-def attack_maps_init():
-    attmsk = [0] * 256
-    attmsk[wP] = 1 << 0
-    attmsk[bP] = 1 << 1
-
-    attmsk[KNIGHT] = 1 << 2
-    attmsk[wN] = 1 << 2
-    attmsk[bN] = 1 << 2
-
-    attmsk[BISHOP] = 1 << 3
-    attmsk[wB] = 1 << 3
-    attmsk[bB] = 1 << 3
-
-    attmsk[ROOK] = 1 << 4
-    attmsk[wR] = 1 << 4
-    attmsk[bR] = 1 << 4
-
-    attmsk[QUEEN] = 1 << 5
-    attmsk[wQ] = 1 << 5
-    attmsk[bQ] = 1 << 5
-
-    attmsk[KING] = 1 << 6
-    attmsk[wK] = 1 << 6
-    attmsk[bK] = 1 << 6
-
-    attmap = [[0] * 64 for i in range(64)]
-    for to_ in range(64):
-        for from_ in range(64):
-            m = 0
-            rook = REACH[ROOK][from_]
-            bishop = REACH[BISHOP][from_]
-            queen = REACH[QUEEN][from_]
-            knight = REACH[KNIGHT][from_]
-            king = REACH[KING][from_]
-
-            if bb_isbiton(knight, to_):
-                m |= attmsk[wN]
-            if bb_isbiton(king, to_):
-                m |= attmsk[wK]
-            if bb_isbiton(rook, to_):
-                m |= attmsk[wR]
-            if bb_isbiton(bishop, to_):
-                m |= attmsk[wB]
-            if bb_isbiton(queen, to_):
-                m |= attmsk[wQ]
-
-            to88 = mapx88(to_)
-            fr88 = mapx88(from_)
-            diff = to88 - fr88
-
-            if diff in [17, 15]:
-                m |= attmsk[wP]
-            elif diff in [-17, -15]:
-                m |= attmsk[bP]
-
-            attmap[to_][from_] = m
-
-    return attmsk, attmap
-
-ATTMSK, ATTMAP = attack_maps_init()
-
-def possible_attack(from_, to_, piece):
-    return 0 != ATTMAP[to_][from_] & ATTMSK[piece]
-
 def norm_kkindex(x, y):
-    if getcol(x) > 3:
+    if chess.square_file(x) > 3:
         x = flip_we(x)
         y = flip_we(y)
 
-    if getrow(x) > 3:
+    if chess.square_rank(x) > 3:
         x = flip_ns(x)
         y = flip_ns(y)
 
-    rowx = getrow(x)
-    colx = getcol(x)
+    rowx = chess.square_rank(x)
+    colx = chess.square_file(x)
 
     if (rowx > colx):
         x = flip_nw_se(x)
         y = flip_nw_se(y)
 
-    rowy = getrow(y)
-    coly = getcol(y)
+    rowy = chess.square_rank(y)
+    coly = chess.square_file(y)
 
     if rowx == colx and rowy > coly:
         x = flip_nw_se(x)
@@ -602,7 +401,7 @@ def init_kkidx():
     for x in range(64):
         for y in range(64):
             # Check if x to y is legal.
-            if not possible_attack(x, y, wK) and x != y:
+            if x != y and not chess.BB_KING_ATTACKS[x] & chess.BB_SQUARES[y]:
                 # Normalize.
                 i, j = norm_kkindex(x, y)
 
@@ -1572,9 +1371,7 @@ iFORBID = tb_FORBID
 iDRAWt = tb_DRAW | 4
 iWMATEt = tb_WMATE | 4
 iBMATEt = tb_BMATE | 4
-iUNKNOWN = tb_UNKNOWN
 
-iUNKNBIT = (1 << 2)
 
 def removepiece(ys, yp, j):
     del ys[j]
@@ -1726,7 +1523,7 @@ Zipinfo = collections.namedtuple("Zipinfo", ["extraoffset", "totalblocks", "bloc
 class PythonTablebases(object):
     """Provides access to Gaviota tablebases using pure Python code."""
 
-    def __init__(self, directory, lzma):
+    def __init__(self, lzma):
         self.lzma = lzma
 
         self.available_tables = {}
@@ -1737,9 +1534,6 @@ class PythonTablebases(object):
         self.block_cache = {}
         self.block_age = 0
 
-        if directory is not None:
-            self.open_directory(directory)
-
     def open_directory(self, directory):
         """Loads *.gtb.cp4* tables from a directory."""
         directory = os.path.abspath(directory)
@@ -1954,7 +1748,7 @@ class PythonTablebases(object):
             if (j > -1) and (ys[j] == xed):
                 # Find capturers (i).
                 for i in range(len(xs)):
-                    if xp[i] == PAWN and (xs[i] == capturer_a or xs[i] == capturer_b):
+                    if xp[i] == chess.PAWN and (xs[i] == capturer_a or xs[i] == capturer_b):
                         epscore = iFORBID
 
                         # Execute capture.
@@ -2101,7 +1895,9 @@ class NativeTablebases(object):
     Has the same interface as :class:`~chess.gaviota.PythonTablebases`.
     """
 
-    def __init__(self, directory, libgtb):
+    def __init__(self, libgtb):
+        self.paths = []
+
         self.libgtb = libgtb
         self.libgtb.tb_init.restype = ctypes.c_char_p
         self.libgtb.tb_restart.restype = ctypes.c_char_p
@@ -2121,10 +1917,6 @@ class NativeTablebases(object):
         if self.libgtb.tb_is_initialized():
             raise RuntimeError("only one gaviota instance can be initialized at a time")
 
-        self.paths = []
-        if directory is not None:
-            self.open_directory(directory)
-
         self._tbcache_restart(1024 * 1024, 50)
 
     def open_directory(self, directory):
@@ -2186,12 +1978,12 @@ class NativeTablebases(object):
         if board.is_insufficient_material():
             return 0
 
-        if chess.popcount(board.occupied) > 5:
-            raise KeyError("gaviota tables support up to 5 pieces, not {0}: {1}".format(chess.popcount(board.occupied), board.fen()))
-
         if board.castling_rights:
             raise KeyError("gaviota tables do not contain positions with castling rights: {0}".format(board.fen()))
 
+        if chess.popcount(board.occupied) > 5:
+            raise KeyError("gaviota tables support up to 5 pieces, not {0}: {1}".format(chess.popcount(board.occupied), board.fen()))
+
         stm = ctypes.c_uint(0 if board.turn == chess.WHITE else 1)
         ep_square = ctypes.c_uint(board.ep_square if board.ep_square else 64)
         castling = ctypes.c_uint(0)
@@ -2230,7 +2022,7 @@ class NativeTablebases(object):
 
         # Probe forbidden.
         if info.value == 3:
-            raise KeyError("gaviota table for {0} marked as forbidden".format(board.fen()))
+            raise MissingTableError("gaviota table for {0} not available".format(board.fen()))
 
         # Probe failed or unknown.
         if not ret or info.value == 7:
@@ -2273,10 +2065,12 @@ def open_tablebases_native(directory, libgtb=None, LibraryLoader=ctypes.cdll):
     :raises: :exc:`RuntimeError` or :exc:`OSError` when libgtb can not be used.
     """
     libgtb = libgtb or ctypes.util.find_library("gtb") or "libgtb.so.1.0.1"
-    return NativeTablebases(directory, LibraryLoader.LoadLibrary(libgtb))
+    tables = NativeTablebases(LibraryLoader.LoadLibrary(libgtb))
+    tables.open_directory(directory)
+    return tables
 
 
-def open_tablebases(directory=None, libgtb=None, LibraryLoader=ctypes.cdll):
+def open_tablebases(directory, libgtb=None, LibraryLoader=ctypes.cdll):
     """
     Opens a collection of tablebases for probing.
 
@@ -2301,4 +2095,6 @@ def open_tablebases(directory=None, libgtb=None, LibraryLoader=ctypes.cdll):
         except ImportError:
             raise ImportError("chess.gaviota requires backports.lzma or libgtb")
 
-    return PythonTablebases(directory, lzma)
+    tables = PythonTablebases(lzma)
+    tables.open_directory(directory)
+    return tables
