@@ -85,19 +85,18 @@ NAG_NOVELTY = 146
 TAG_REGEX = re.compile(r"^\[([A-Za-z0-9_]+)\s+\"(.*)\"\]\s*$")
 
 MOVETEXT_REGEX = re.compile(r"""
-    (%.*?[\n\r])
-    |(\{.*)
-    |(\$[0-9]+)
-    |(\()
-    |(\))
-    |(\*|1-0|0-1|1/2-1/2)
-    |(
+    (
         [NBKRQ]?[a-h]?[1-8]?[\-x]?[a-h][1-8](?:=?[nbrqkNBRQK])?
         |[PNBRQK]?@[a-h][1-8]
         |--
         |O-O(?:-O)?
         |0-0(?:-0)?
     )
+    |(\{.*)
+    |(\$[0-9]+)
+    |(\()
+    |(\))
+    |(\*|1-0|0-1|1/2-1/2)
     |([\?!]{1,2})
     """, re.DOTALL | re.VERBOSE)
 
@@ -135,11 +134,21 @@ class GameNode(object):
     def san(self):
         """
         Gets the standard algebraic notation of the move leading to this node.
+        See :func:`chess.Board.san()`.
 
         Do not call this on the root node.
         """
         return self.parent.board().san(self.move)
 
+    def uci(self, chess960=None):
+        """
+        Gets the UCI notation of the move leading to this node.
+        See :func:`chess.Board.uci()`.
+
+        Do not call this on the root node.
+        """
+        return self.parent.board().san(self.move, chess960=chess960)
+
     def root(self):
         """Gets the root node, i.e. the game."""
         node = self
@@ -395,8 +404,8 @@ class Game(GameNode):
         """
         Gets the starting position of the game.
 
-        Unless the `SetUp` and `FEN` header tags are set this is the default
-        starting position.
+        Unless the ``FEN`` header tag is set this is the default starting
+        position (for the ``Variant``).
         """
         chess960 = self.headers.get("Variant", "").lower() in [
             "chess960",
@@ -409,15 +418,15 @@ class Game(GameNode):
             from chess.variant import find_variant
             VariantBoard = find_variant(self.headers["Variant"])
 
-        fen = self.headers.get("FEN") if self.headers.get("SetUp", "1") == "1" else None
-        board = VariantBoard(fen or VariantBoard.starting_fen, chess960=chess960)
+        fen = self.headers.get("FEN", VariantBoard.starting_fen)
+        board = VariantBoard(fen, chess960=chess960)
         board.chess960 = board.chess960 or board.has_chess960_castling_rights()
         return board
 
     def setup(self, board):
         """
-        Setup a specific starting position. This sets (or resets) the *SetUp*,
-        *FEN* and *Variant* header tags.
+        Setup a specific starting position. This sets (or resets) the
+        ``FEN``, ``SetUp``, and ``Variant`` header tags.
         """
         try:
             fen = board.fen()
@@ -564,7 +573,7 @@ class GameModelCreator(BaseVisitor):
     def __init__(self):
         self.game = Game()
 
-        self.variation_stack = collections.deque([self.game])
+        self.variation_stack = [self.game]
         self.starting_comment = ""
         self.in_variation = False
 
@@ -586,7 +595,7 @@ class GameModelCreator(BaseVisitor):
             self.game.headers["Result"] = result
 
     def visit_comment(self, comment):
-        if self.in_variation or (not self.variation_stack[-1].parent and self.variation_stack[-1].is_end()):
+        if self.in_variation or (self.variation_stack[-1].parent is None and self.variation_stack[-1].is_end()):
             # Add as a comment for the current node if in the middle of
             # a variation. Add as a comment for the game, if the comment
             # starts before any move.
@@ -770,12 +779,11 @@ def read_game(handle, Visitor=GameModelCreator):
     By using text mode the parser does not need to handle encodings. It is the
     callers responsibility to open the file with the correct encoding.
     PGN files are ASCII or UTF-8 most of the time. So the following should
-    cover most relevant cases (ASCII, UTF-8 without BOM, UTF-8 with BOM,
-    UTF-8 with encoding errors).
+    cover most relevant cases (ASCII, UTF-8, UTF-8 with BOM).
 
-    >>> pgn = open("data/pgn/kasparov-deep-blue-1997.pgn", encoding="utf-8-sig", errors="surrogateescape")
+    >>> pgn = open("data/pgn/kasparov-deep-blue-1997.pgn", encoding="utf-8-sig")
 
-    Use `StringIO` to parse games from a string.
+    Use :class:`~io.StringIO` to parse games from a string.
 
     >>> pgn_string = "1. e4 e5 2. Nf3 *"
     >>>
@@ -810,7 +818,7 @@ def read_game(handle, Visitor=GameModelCreator):
     # Parse game headers.
     while line:
         # Skip empty lines and comments.
-        if line.isspace() or line.lstrip().startswith("%"):
+        if line.isspace() or line.startswith("%"):
             line = handle.readline()
             continue
 
@@ -838,15 +846,20 @@ def read_game(handle, Visitor=GameModelCreator):
 
     # Movetext parser state.
     try:
-        board_stack = collections.deque([dummy_game.board()])
+        board_stack = [dummy_game.board()]
     except ValueError as error:
         visitor.handle_error(error)
-        board_stack = collections.deque([chess.Board()])
+        board_stack = [chess.Board()]
 
     # Parse movetext.
     while line:
         read_next_line = True
 
+        if line.startswith("%"):
+            # Ignore comments.
+            line = handle.readline()
+            continue
+
         # An empty line is the end of a game.
         if found_content and line.isspace():
             if found_game:
@@ -858,11 +871,6 @@ def read_game(handle, Visitor=GameModelCreator):
         for match in MOVETEXT_REGEX.finditer(line):
             token = match.group(0)
 
-            if token.startswith("%"):
-                # Ignore the rest of the line.
-                line = handle.readline()
-                continue
-
             if not found_game:
                 found_game = True
                 visitor.begin_game()
@@ -907,19 +915,16 @@ def read_game(handle, Visitor=GameModelCreator):
                 visitor.visit_nag(NAG_SPECULATIVE_MOVE)
             elif token == "?!":
                 visitor.visit_nag(NAG_DUBIOUS_MOVE)
-            elif token == "(":
-                if board_stack[-1].move_stack:
-                    visitor.begin_variation()
+            elif token == "(" and board_stack[-1].move_stack:
+                visitor.begin_variation()
 
-                    board = board_stack[-1].copy()
-                    board.pop()
-                    board_stack.append(board)
-            elif token == ")":
-                # Found a close variation token. Always leave at least the
-                # root node on the stack.
-                if len(board_stack) > 1:
-                    visitor.end_variation()
-                    board_stack.pop()
+                board = board_stack[-1].copy()
+                board.pop()
+                board_stack.append(board)
+            elif token == ")" and len(board_stack) > 1:
+                # Always leave at least the root node on the stack.
+                visitor.end_variation()
+                board_stack.pop()
             elif token in ["1-0", "0-1", "1/2-1/2", "*"] and len(board_stack) == 1:
                 # Found a result token.
                 found_content = True
@@ -1050,7 +1055,7 @@ def scan_offsets(handle):
     later. This is just like :func:`~chess.pgn.scan_headers()` but more
     efficient if you do not actually need the header information.
 
-    The PGN standard requires each game to start with an *Event*-tag. So does
+    The PGN standard requires each game to start with an ``Event`` tag. So does
     this scanner.
     """
     in_comment = False
