@@ -467,12 +467,6 @@ class HordeBoard(chess.Board):
         return status
 
 
-THREE_CHECK_POLYGLOT_RANDOM_ARRAY = [
-    0x1d6dc0ee61ce803e, 0xc6284b653d38e96a,
-    0x803f5fb0d2f97fae, 0xb183ccc9e73df9ed,
-    0xfdeef11602d6b443, 0x1b0ce4198b3801a6,
-]
-
 class ThreeCheckBoard(chess.Board):
 
     aliases = ["Three-check", "Three check", "Threecheck", "Three check chess"]
@@ -511,22 +505,6 @@ class ThreeCheckBoard(chess.Board):
     def is_insufficient_material(self):
         return self.occupied == self.kings
 
-    def zobrist_hash(self, array=None, three_check_array=None):
-        zobrist_hash = super(ThreeCheckBoard, self).zobrist_hash(array)
-
-        if three_check_array is None:
-            three_check_array = THREE_CHECK_POLYGLOT_RANDOM_ARRAY
-
-        white_count = min(3, 3 - self.remaining_checks[chess.WHITE])
-        black_count = min(3, 3 - self.remaining_checks[chess.BLACK])
-
-        if white_count > 0:
-            zobrist_hash ^= three_check_array[black_count - 1]
-        if black_count > 0:
-            zobrist_hash ^= three_check_array[white_count + 2]
-
-        return zobrist_hash
-
     def set_epd(self, epd):
         # Split into 5 or 6 parts.
         parts = epd.strip().rstrip(";").split(None, 5)
@@ -592,6 +570,19 @@ class ThreeCheckBoard(chess.Board):
     def is_variant_win(self):
         return self.remaining_checks[self.turn] <= 0 and self.remaining_checks[not self.turn] > 0
 
+    def is_irreversible(move):
+        if super(ThreeCheckBoard, self).is_irreversible(move):
+            return True
+
+        self.push(move)
+        gives_check = self.is_check()
+        self.pop()
+        return gives_check
+
+    def _transposition_key(self):
+        return (super(ThreeCheckBoard, self)._transposition_key(),
+                self.remaining_checks[WHITE], self.remaining_checks[BLACK])
+
     def copy(self, stack=True):
         board = super(ThreeCheckBoard, self).copy(stack=stack)
         board.remaining_checks[chess.WHITE] = self.remaining_checks[chess.WHITE]
@@ -691,6 +682,10 @@ class CrazyhouseBoard(chess.Board):
                 castling_rights & chess.BB_SQUARES[move.from_square] or
                 castling_rights & chess.BB_SQUARES[move.to_square])
 
+    def _transposition_key(self):
+        return (super(CrazyhouseBoard, self)._transposition_key(),
+                str(self.pockets[chess.WHITE]), str(self.pockets[chess.BLACK]))
+
     def legal_drop_squares_mask(self):
         king = self.king(self.turn)
         if king is None:
@@ -743,8 +738,6 @@ class CrazyhouseBoard(chess.Board):
             super(CrazyhouseBoard, self).generate_legal_moves(from_mask, to_mask),
             self.generate_legal_drops(from_mask & to_mask))
 
-    # TODO: zobrist_hash
-
     def parse_san(self, san):
         if "@" in san:
             uci = san.rstrip("+# ")
